"""
__version__.py
~~~~~~~~~~~~~~

Information about the current version of birp package.
"""

__title__ = "birp"
__description__ = "birp — большой русский питон (BIg Russian Python)"
__version__ = "0.2.0"
__author__ = "evtn"
__author_email__ = "g@evtn.ru"
__license__ = "MIT"
__url__ = "https://github.com/evtn/birp"
