from argparse import ArgumentParser as ПарсерАргументов

аргпарсер = ПарсерАргументов(
    description="birp - большой русский питон. Позволяет писать программы на русском языке, как настоящий патриот!!!!", 
    prog="birp[.reverse]"
)
аргпарсер.add_argument(
    "-ф", "--файлы", "-f", "--files", nargs="+", help="Список файлов, которые необъодимо перевести"
)
аргпарсер.add_argument(
    "-п",
    "--переводы",
    help="ЯСНО*-файл с дополнительными переводами имён (должен содержать объект вида {русское_имя: английское_имя})\n*ЯваСценарная Нотация Объектов",
)
аргпарсер.add_argument(
    "-д",
    "--дерево",
    dest="tree",
    action="store_true",
    help="Вывод дерева в отдельный файл",
)
